@@ -1,4 +1,3 @@
-from gettext import gettext as _
 from django.conf import settings
 from drf_spectacular.utils import extend_schema_field
 from drf_spectacular.types import OpenApiTypes
@@ -11,9 +10,9 @@ from pulp_ansible.app import models
 class CollectionSerializer(serializers.ModelSerializer):
     """A serializer for a Collection."""
 
-    created_at = serializers.DateTimeField(source="collection.pulp_created")
-    updated_at = serializers.DateTimeField(source="collection.pulp_last_updated")
-    deprecated = serializers.BooleanField(source="collection.deprecated")
+    deprecated = serializers.BooleanField()
+    created_at = serializers.SerializerMethodField()
+    updated_at = serializers.SerializerMethodField()
     href = serializers.SerializerMethodField()
 
     versions_url = serializers.SerializerMethodField()
@@ -22,24 +21,15 @@ class CollectionSerializer(serializers.ModelSerializer):
     class Meta:
         fields = (
             "href",
-            "created_at",
-            "updated_at",
             "namespace",
             "name",
             "deprecated",
             "versions_url",
             "highest_version",
+            "created_at",
+            "updated_at",
         )
-        model = models.CollectionVersion
-
-    def update(self, obj, validated_data):
-        """Update Collection."""
-        collection_data = validated_data.pop("collection")
-        if "deprecated" not in collection_data or len(collection_data.keys()) > 1:
-            raise serializers.ValidationError(_("Only `deprecated` could be updated."))
-        collection = models.Collection.objects.filter(pk=obj.collection.pk)
-        collection.update(**collection_data)
-        return super().update(obj, validated_data)
+        model = models.Collection
 
     def get_href(self, obj):
         """Get href."""
@@ -55,19 +45,30 @@ class CollectionSerializer(serializers.ModelSerializer):
             kwargs={"path": self.context["path"], "namespace": obj.namespace, "name": obj.name},
         )
 
+    def get_created_at(self, obj):
+        """Get the timestamp of the lowest version CollectionVersion's created timestamp."""
+        collection = self.context["lowest_versions"][obj.pk]
+        return collection.pulp_created
+
+    def get_updated_at(self, obj):
+        """Get the timestamp of the highest version CollectionVersion's created timestamp."""
+        collection = self.context["highest_versions"][obj.pk]
+        return collection.pulp_created
+
     @extend_schema_field(OpenApiTypes.OBJECT)
     def get_highest_version(self, obj):
         """Get a highest version and its link."""
+        collection = self.context["highest_versions"][obj.pk]
         href = reverse(
             "collection-versions-detail",
             kwargs={
                 "path": self.context["path"],
-                "namespace": obj.namespace,
-                "name": obj.name,
-                "version": obj.version,
+                "namespace": collection.namespace,
+                "name": collection.name,
+                "version": collection.version,
             },
         )
-        return {"href": href, "version": obj.version}
+        return {"href": href, "version": str(collection.version)}
 
 
 class CollectionVersionListSerializer(serializers.ModelSerializer):
@@ -78,7 +79,7 @@ class CollectionVersionListSerializer(serializers.ModelSerializer):
     updated_at = serializers.DateTimeField(source="collection.pulp_last_updated")
 
     class Meta:
-        fields = ("version", "certification", "href", "created_at", "updated_at")
+        fields = ("version", "href", "created_at", "updated_at")
         model = models.CollectionVersion
 
     def get_href(self, obj):
