@@ -1,14 +1,17 @@
+from collections import defaultdict, namedtuple
 from datetime import datetime
 from gettext import gettext as _
 import semantic_version
 
-from django.db.models import Q
+from django.db.models import Exists, OuterRef
 from django.shortcuts import get_object_or_404
+from django.utils.datastructures import MultiValueDictKeyError
 from django.utils.dateparse import parse_datetime
 from drf_spectacular.utils import OpenApiParameter, extend_schema
-from rest_framework.decorators import action
 from rest_framework import mixins
+from rest_framework.exceptions import ValidationError
 from rest_framework.response import Response
+from rest_framework.reverse import reverse
 from rest_framework import serializers
 from rest_framework import status as http_status
 from rest_framework import viewsets
@@ -16,7 +19,6 @@ from rest_framework import viewsets
 from pulpcore.plugin.exceptions import DigestValidationError
 from pulpcore.plugin.models import PulpTemporaryFile, Content, ContentArtifact
 from pulpcore.plugin.serializers import AsyncOperationResponseSerializer
-from rest_framework.reverse import reverse
 
 from pulp_ansible.app.galaxy.v3.exceptions import ExceptionHandlerMixin
 from pulp_ansible.app.galaxy.v3.serializers import (
@@ -25,7 +27,13 @@ from pulp_ansible.app.galaxy.v3.serializers import (
     CollectionVersionDocsSerializer,
     CollectionVersionListSerializer,
 )
-from pulp_ansible.app.models import AnsibleDistribution, CollectionVersion, CollectionImport
+from pulp_ansible.app.models import (
+    AnsibleCollectionDeprecated,
+    AnsibleDistribution,
+    Collection,
+    CollectionVersion,
+    CollectionImport,
+)
 from pulp_ansible.app.serializers import (
     CollectionOneShotSerializer,
     CollectionImportDetailSerializer,
@@ -36,23 +44,31 @@ from pulp_ansible.app.galaxy.v3.pagination import LimitOffsetPagination
 from pulp_ansible.app.viewsets import CollectionVersionFilter
 
 
+CollectionTuple = namedtuple("CollectionTuple", ["namespace", "name", "version", "pulp_created"])
+
+
 class AnsibleDistributionMixin:
     """
     A mixin for ViewSets that use AnsibleDistribution.
     """
 
     @staticmethod
-    def get_distro_content(path):
-        """Returns distribution content."""
+    def get_repository_version(path):
+        """Returns repository version."""
         distro = get_object_or_404(AnsibleDistribution, base_path=path)
         if distro.repository_version:
-            return distro.repository_version.content
-        else:
-            repo_version = distro.repository.latest_version()
-            if repo_version is None:
-                return Content.objects.none()
-            else:
-                return repo_version.content
+            return distro.repository_version
+
+        return distro.repository.latest_version()
+
+    @staticmethod
+    def get_distro_content(path):
+        """Returns distribution content."""
+        repo_version = AnsibleDistributionMixin.get_repository_version(path)
+        if repo_version is None:
+            return Content.objects.none()
+
+        return repo_version.content
 
     def get_serializer_context(self):
         """Inserts distribution path to a serializer context."""
@@ -109,31 +125,61 @@ class CollectionViewSet(
     serializer_class = CollectionSerializer
     pagination_class = LimitOffsetPagination
 
+    def filter_queryset(self, queryset):
+        """
+        Filter Repository related fields.
+        """
+        queryset = super().filter_queryset(queryset)
+
+        try:
+            user_deprecated_value = self.request.query_params["deprecated"]
+        except MultiValueDictKeyError:
+            pass
+        else:
+            if user_deprecated_value.lower() in ["true", "yes", "1"]:
+                queryset = queryset.filter(deprecated=True)
+            elif user_deprecated_value.lower() in ["false", "no", "0"]:
+                queryset = queryset.filter(deprecated=False)
+            else:
+                raise ValidationError("Cannot parse value of `deprecated` GET parameter")
+
+        return queryset
+
     def get_queryset(self):
         """
-        Returns a CollectionVersions queryset for specified distribution.
+        Returns a Collections queryset for specified distribution.
         """
-        distro_content = self.get_distro_content(self.kwargs["path"])
+        repo_version = self.get_repository_version(self.kwargs["path"])
+        deprecated_query = AnsibleCollectionDeprecated.objects.filter(
+            collection=OuterRef("pk"), repository_version=repo_version
+        )
+        collections = Collection.objects.filter(versions__in=repo_version.content).distinct()
+        collections = collections.annotate(deprecated=Exists(deprecated_query))
 
-        versions = CollectionVersion.objects.filter(pk__in=distro_content).values_list(
+        versions_qs = CollectionVersion.objects.filter(pk__in=repo_version.content).values_list(
             "collection_id",
+            "namespace",
+            "name",
             "version",
+            "pulp_created",
         )
 
-        collection_versions = {}
-        for collection_id, version in versions:
-            value = collection_versions.get(str(collection_id))
-            if not value or semantic_version.Version(version) > semantic_version.Version(value):
-                collection_versions[str(collection_id)] = version
+        highest_versions = defaultdict(
+            lambda: CollectionTuple(None, None, semantic_version.Version("0.0.0"), None)
+        )
+        lowest_versions = defaultdict(
+            lambda: CollectionTuple(None, None, semantic_version.Version("100000000000.0.0"), None)
+        )
+        for collection_id, namespace, name, version, pulp_created in versions_qs:
+            version_to_consider = semantic_version.Version(version)
+            collection_tuple = CollectionTuple(namespace, name, version_to_consider, pulp_created)
+            if version_to_consider > highest_versions[collection_id].version:
+                highest_versions[collection_id] = collection_tuple
+            if version_to_consider < lowest_versions[collection_id].version:
+                lowest_versions[collection_id] = collection_tuple
 
-        if not collection_versions.items():
-            return CollectionVersion.objects.none()
-
-        query_params = Q()
-        for collection_id, version in collection_versions.items():
-            query_params |= Q(collection_id=collection_id, version=version)
-
-        collections = CollectionVersion.objects.select_related("collection").filter(query_params)
+        self.highest_versions_context = highest_versions  # needed by get__serializer_context
+        self.lowest_versions_context = lowest_versions  # needed by get__serializer_context
         return collections
 
     def get_object(self):
@@ -141,20 +187,39 @@ class CollectionViewSet(
         Returns a Collection object.
         """
         queryset = self.filter_queryset(self.get_queryset())
-
         return get_object_or_404(
             queryset, namespace=self.kwargs["namespace"], name=self.kwargs["name"]
         )
 
+    def get_serializer_context(self, *args, **kwargs):
+        """
+        Return the serializer context.
+
+        This uses super() but also adds in the "highest_versions" data from get_queryset()
+        """
+        super_data = super().get_serializer_context()
+        super_data["highest_versions"] = self.highest_versions_context
+        super_data["lowest_versions"] = self.lowest_versions_context
+        return super_data
+
     def update(self, request, *args, **kwargs):
         """
         Update a Collection object.
         """
-        instance = self.get_object()
-        serializer = self.get_serializer(instance, data=request.data, partial=True)
+        repo_version = self.get_repository_version(self.kwargs["path"])
+        collection = self.get_object()
+        serializer = self.get_serializer(collection, data=request.data, partial=True)
         serializer.is_valid(raise_exception=True)
-        serializer.save()
-        return Response(serializer.data)
+        deprecated_value = serializer.validated_data["deprecated"]
+        if deprecated_value:
+            AnsibleCollectionDeprecated.objects.get_or_create(
+                repository_version=repo_version, collection=collection
+            )
+        else:
+            AnsibleCollectionDeprecated.objects.filter(
+                repository_version=repo_version, collection=collection
+            ).delete()
+        return Response()
 
 
 class CollectionUploadViewSet(
@@ -166,7 +231,7 @@ class CollectionUploadViewSet(
 
     authentication_classes = []
     permission_classes = []
-    serializer_class = CollectionOneShotSerializer
+    serializer_class = CollectionSerializer
 
     @extend_schema(
         description="Create an artifact and trigger an asynchronous task to create "
@@ -180,7 +245,7 @@ class CollectionUploadViewSet(
         Dispatch a Collection creation task.
         """
         distro = get_object_or_404(AnsibleDistribution, base_path=path)
-        serializer = self.get_serializer(data=request.data, context={"request": request})
+        serializer = CollectionOneShotSerializer(data=request.data, context={"request": request})
         serializer.is_valid(raise_exception=True)
 
         expected_digests = {}
@@ -260,16 +325,6 @@ class CollectionVersionViewSet(
         serializer = CollectionVersionListSerializer(queryset, many=True, context=context)
         return Response(serializer.data)
 
-    @action(methods=["PUT"], detail=True, url_path="certified")
-    def set_certified(self, request, *args, **kwargs):
-        """
-        Set collection version certified status.
-        """
-        obj = self.get_object()
-        obj.certification = request.data["certification"]
-        obj.save()
-        return Response({})
-
 
 class CollectionVersionDocsViewSet(
     CollectionVersionRetrieveMixin,
