@@ -1,6 +1,7 @@
 from gettext import gettext as _
 
 from django.conf import settings
+from jsonschema import Draft7Validator
 from rest_framework import serializers
 
 from pulpcore.plugin.serializers import (
@@ -10,6 +11,7 @@ from pulpcore.plugin.serializers import (
     RepositorySerializer,
     SingleArtifactContentSerializer,
     RepositoryVersionDistributionSerializer,
+    validate_unknown_fields,
 )
 
 from .models import (
@@ -23,6 +25,7 @@ from .models import (
     Role,
     Tag,
 )
+from pulp_ansible.app.schema import COPY_CONFIG_SCHEMA
 from pulp_ansible.app.tasks.utils import parse_collections_requirements_file
 
 
@@ -116,7 +119,7 @@ class CollectionRemoteSerializer(RemoteSerializer):
         """
         Validate collection remote data.
 
-            - url: to ensure it does not end with slashes.
+            - url: to ensure it ends with slashes.
             - requirements_file: to ensure it is a valid yaml file.
 
         Args:
@@ -129,26 +132,23 @@ class CollectionRemoteSerializer(RemoteSerializer):
             rest_framework.serializers.ValidationError: If the url or requirements_file is invalid
 
         """
+
+        def _validate_url(url):
+            """Based on https://git.io/JTMAA."""
+            if not url.endswith("/") and url != "https://galaxy.ansible.com":
+                raise serializers.ValidationError(
+                    _("Invalid URL {url}. Ensure the URL ends '/'.").format(url=data["url"])
+                )
+
         data = super().validate(data)
 
+        _validate_url(data["url"])
+
         if data.get("requirements_file"):
-            parsed_file = parse_collections_requirements_file(data["requirements_file"])
-            urls_to_validate = [data["url"]]
-            for name, version, source in parsed_file:
-                if source:
-                    urls_to_validate.append(source)
-
-            for url in urls_to_validate:
-                if not url.endswith("/v2/collections") and not url.endswith("/v3/collections"):
-                    raise serializers.ValidationError(
-                        _(
-                            "Invalid URL {url}. ".format(url=data["url"])
-                            + "Ensure the URL ends in either '/v2/collections' or '/v3/collections'"
-                        )
-                    )
-
-        if data["url"].endswith("/"):
-            raise serializers.ValidationError(_("url should not end with '/'"))
+            collections = parse_collections_requirements_file(data["requirements_file"])
+            for collection in collections:
+                if collection[2]:
+                    _validate_url(collection[2])
 
         return data
 
@@ -211,7 +211,7 @@ class AnsibleDistributionSerializer(RepositoryVersionDistributionSerializer):
         """
         Get client_url.
         """
-        return "{hostname}/pulp_ansible/galaxy/{base_path}".format(
+        return "{hostname}/pulp_ansible/galaxy/{base_path}/".format(
             hostname=settings.ANSIBLE_API_HOSTNAME, base_path=obj.base_path
         )
 
@@ -312,8 +312,6 @@ class CollectionVersionSerializer(SingleArtifactContentSerializer, ContentChecks
         help_text=_("The URL to the collection issue tracker."), allow_blank=True, max_length=2000
     )
 
-    certification = serializers.CharField(help_text=_("Indicates that the version is certified"))
-
     license = serializers.ListField(
         help_text=_("A list of licenses for content inside of a collection."),
         child=serializers.CharField(max_length=32),
@@ -333,12 +331,6 @@ class CollectionVersionSerializer(SingleArtifactContentSerializer, ContentChecks
 
     version = serializers.CharField(help_text=_("The version of the collection."), max_length=32)
 
-    deprecated = serializers.BooleanField(
-        source="collection.deprecated",
-        help_text=_("Whether or not the collection has been deprecated."),
-        read_only=True,
-    )
-
     class Meta:
         fields = (
             tuple(set(SingleArtifactContentSerializer.Meta.fields) - {"relative_path"})
@@ -353,14 +345,12 @@ class CollectionVersionSerializer(SingleArtifactContentSerializer, ContentChecks
                 "documentation",
                 "homepage",
                 "issues",
-                "certification",
                 "license",
                 "name",
                 "namespace",
                 "repository",
                 "tags",
                 "version",
-                "deprecated",
             )
         )
         model = CollectionVersion
@@ -393,3 +383,39 @@ class CollectionImportDetailSerializer(CollectionImportListSerializer):
 
     class Meta(CollectionImportListSerializer.Meta):
         fields = CollectionImportListSerializer.Meta.fields + ("error", "messages")
+
+
+class CopySerializer(serializers.Serializer):
+    """
+    A serializer for Content Copy API.
+    """
+
+    config = serializers.JSONField(
+        help_text=_("A JSON document describing sources, destinations, and content to be copied"),
+    )
+
+    def validate(self, data):
+        """
+        Validate that the Serializer contains valid data.
+
+        Set the AnsibleRepository based on the RepositoryVersion if only the latter is provided.
+        Set the RepositoryVersion based on the AnsibleRepository if only the latter is provided.
+        Convert the human-friendly names of the content types into what Pulp needs to query on.
+        """
+        super().validate(data)
+
+        if hasattr(self, "initial_data"):
+            validate_unknown_fields(self.initial_data, self.fields)
+
+        if "config" in data:
+            validator = Draft7Validator(COPY_CONFIG_SCHEMA)
+
+            err = []
+            for error in sorted(validator.iter_errors(data["config"]), key=str):
+                err.append(error.message)
+            if err:
+                raise serializers.ValidationError(
+                    _("Provided copy criteria is invalid:'{}'".format(err))
+                )
+
+        return data
