@@ -1,10 +1,11 @@
 import asyncio
+from aiohttp.client_exceptions import ClientResponseError
 from gettext import gettext as _
 import json
 import logging
 import math
 import tarfile
-from urllib.parse import urlparse, urlunparse
+from urllib.parse import urljoin, urlparse, urlunparse
 
 from django.db import transaction
 from django.urls import reverse
@@ -20,7 +21,6 @@ from pulpcore.plugin.models import (
     ProgressReport,
     PulpTemporaryFile,
     Remote,
-    Repository,
 )
 from pulpcore.plugin.stages import (
     ArtifactDownloader,
@@ -39,6 +39,8 @@ import semantic_version as semver
 
 from pulp_ansible.app.constants import PAGE_SIZE
 from pulp_ansible.app.models import (
+    AnsibleCollectionDeprecated,
+    AnsibleRepository,
     Collection,
     CollectionImport,
     CollectionRemote,
@@ -47,7 +49,6 @@ from pulp_ansible.app.models import (
 )
 from pulp_ansible.app.serializers import CollectionVersionSerializer
 from pulp_ansible.app.tasks.utils import (
-    get_api_version,
     get_page_url,
     parse_metadata,
     parse_collections_requirements_file,
@@ -71,7 +72,7 @@ def sync(remote_pk, repository_pk, mirror):
 
     """
     remote = CollectionRemote.objects.get(pk=remote_pk)
-    repository = Repository.objects.get(pk=repository_pk)
+    repository = AnsibleRepository.objects.get(pk=repository_pk)
 
     if not remote.url:
         raise ValueError(_("A CollectionRemote must have a 'url' specified to synchronize."))
@@ -150,7 +151,7 @@ def import_collection(
     CreatedResource.objects.create(content_object=collection_version)
 
     if repository_pk:
-        repository = Repository.objects.get(pk=repository_pk)
+        repository = AnsibleRepository.objects.get(pk=repository_pk)
         content_q = CollectionVersion.objects.filter(pk=collection_version.pk)
         with repository.new_version() as new_version:
             new_version.add_content(content_q)
@@ -252,7 +253,7 @@ class AnsibleDeclarativeVersion(DeclarativeVersion):
             ArtifactSaver(),
             QueryExistingContents(),
             DocsBlobDownloader(),
-            CollectionContentSaver(),
+            CollectionContentSaver(new_version),
             RemoteArtifactSaver(),
             ResolveContentFutures(),
         ]
@@ -284,7 +285,8 @@ class CollectionSyncFirstStage(Stage):
         """
         Build and emit `DeclarativeContent` from the ansible metadata.
         """
-        with ProgressReport(message="Parsing Collection Metadata", code="parsing.metadata") as pb:
+        msg = "Parsing CollectionVersion Metadata"
+        with ProgressReport(message=msg, code="parsing.metadata") as pb:
             async for metadata in self._fetch_collections():
 
                 url = metadata["download_url"]
@@ -313,7 +315,10 @@ class CollectionSyncFirstStage(Stage):
                     deferred_download=self.deferred_download,
                 )
 
-                extradata = dict(docs_blob_url=metadata["docs_blob_url"])
+                extradata = dict(
+                    docs_blob_url=metadata["docs_blob_url"],
+                    deprecated=metadata["deprecated"],
+                )
 
                 d_content = DeclarativeContent(
                     content=collection_version,
@@ -331,19 +336,83 @@ class CollectionSyncFirstStage(Stage):
             async generator: dicts that represent collections from galaxy api
 
         """
-        page_count = 1
         remote = self.remote
         collection_info = self.collection_info
 
-        def _get_url(page, api_version):
-            if collection_info:
+        async def _get_collection_api(root):
+            """
+            Returns the collection api path and api version.
+
+            Based on https://git.io/JTMxE.
+            """
+            if root == "https://galaxy.ansible.com" or root == "https://galaxy.ansible.com/":
+                root = "https://galaxy.ansible.com/api/"
+
+            downloader = remote.get_downloader(url=root)
+
+            try:
+                api_data = parse_metadata(await downloader.run())
+            except (json.decoder.JSONDecodeError, ClientResponseError):
+                if root.endswith("/api/"):
+                    raise
+
+                root = urljoin(root, "api/")
+                downloader = remote.get_downloader(url=root)
+                api_data = parse_metadata(await downloader.run())
+
+            if "available_versions" not in api_data:
+                raise RuntimeError(_("Could not find 'available_versions' at {}").format(root))
+
+            if "v3" in api_data.get("available_versions", {}):
+                self.api_version = 3
+            elif "v2" in api_data.get("available_versions", {}):
+                self.api_version = 2
+            else:
+                raise RuntimeError(_("Unsupported API versions at {}").format(root))
+
+            endpoint = f"{root}v{self.api_version}/collections/"
+
+            return endpoint, self.api_version
+
+        async def _get_url(page, versions_url=None):
+            if collection_info and not versions_url:
                 name, version, source = collection_info[page - 1]
                 namespace, name = name.split(".")
                 root = source or remote.url
-                url = f"{root.rstrip('/')}/{namespace}/{name}"
+                api_endpoint = (await _get_collection_api(root))[0]
+                url = f"{api_endpoint}{namespace}/{name}/"
                 return url
 
-            return get_page_url(remote.url, api_version, page)
+            if not versions_url:
+                api_endpoint, api_version = await _get_collection_api(remote.url)
+                return get_page_url(api_endpoint, api_version, page)
+
+            if not self.api_version:
+                await _get_collection_api(remote.url)
+
+            return get_page_url(versions_url, self.api_version, page)
+
+        async def _loop_through_pages(not_done, versions_url=None):
+            """
+            Loop through API pagination.
+            """
+            url = await _get_url(1, versions_url)
+            downloader = remote.get_downloader(url=url)
+            data = parse_metadata(await downloader.run())
+
+            count = data.get("count") or data.get("meta", {}).get("count", 1)
+            if collection_info and not versions_url:
+                count = len(collection_info)
+                page_count = count
+            else:
+                page_count = math.ceil(float(count) / float(PAGE_SIZE))
+
+            for page in range(1, page_count + 1):
+                url = await _get_url(page, versions_url)
+                downloader = remote.get_downloader(url=url)
+                not_done.add(downloader.run())
+
+            return count
 
         def _build_url(path_or_url):
             """Check value and turn it into a url using remote.url if it's a relative path."""
@@ -354,25 +423,25 @@ class CollectionSyncFirstStage(Stage):
             else:
                 return path_or_url
 
+        def _add_collection_level_metadata(data, additional_metadata):
+            """Additional metadata at collection level to be sent through stages."""
+            name = data["collection"]["name"]
+            namespace = data["namespace"]["name"]
+            metadata = additional_metadata.get(f"{namespace}_{name}", {})
+            data["deprecated"] = metadata.get("deprecated")
+
+        def _add_collection_version_level_metadata(data, additional_metadata):
+            """Additional metadata at collection version level to be sent through stages."""
+            metadata = additional_metadata.get(_build_url(data["href"]), {})
+            data["docs_blob_url"] = metadata.get("docs_blob_url")
+
         progress_data = dict(message="Parsing Galaxy Collections API", code="parsing.collections")
         with ProgressReport(**progress_data) as progress_bar:
-            api_version = get_api_version(remote.url)
-            url = _get_url(page_count, api_version)
-            downloader = remote.get_downloader(url=url)
-            initial_data = parse_metadata(await downloader.run())
-
-            _count = initial_data.get("count") or initial_data.get("meta", {}).get("count", 1)
-            count = len(self.collection_info) * PAGE_SIZE or _count
-            page_count = math.ceil(float(count) / float(PAGE_SIZE))
+            not_done = set()
+            count = await _loop_through_pages(not_done)
             progress_bar.total = count
             progress_bar.save()
 
-            # Concurrent downloads are limited by aiohttp...
-            not_done = set()
-            for page in range(1, page_count + 1):
-                downloader = remote.get_downloader(url=_get_url(page, api_version))
-                not_done.add(downloader.run())
-
             additional_metadata = {}
 
             while not_done:
@@ -381,17 +450,30 @@ class CollectionSyncFirstStage(Stage):
                 for item in done:
                     data = parse_metadata(item.result())
 
-                    if api_version < 3:
-                        results = data.get("results", [data])
+                    if "data" in data:  # api v3
+                        results = data["data"]
+                    elif "results" in data:  # api v2
+                        results = data["results"]
                     else:
-                        results = data.get("data", [data])
+                        results = [data]
 
                     for result in results:
                         download_url = result.get("download_url")
 
+                        if result.get("deprecated"):
+                            name = result["name"]
+                            try:
+                                namespace = result["namespace"]["name"]  # api v3
+                            except TypeError:
+                                namespace = result["namespace"]  # api v2
+                            additional_metadata[f"{namespace}_{name}"] = {
+                                "deprecated": result["deprecated"]
+                            }
+
                         if result.get("versions_url"):
                             versions_url = _build_url(result.get("versions_url"))
-                            not_done.update([remote.get_downloader(url=versions_url).run()])
+                            await _loop_through_pages(not_done, versions_url)
+                            progress_bar.increment()
 
                         if result.get("version") and not download_url:
                             version_url = _build_url(result["href"])
@@ -401,10 +483,9 @@ class CollectionSyncFirstStage(Stage):
                             }
 
                         if download_url:
-                            metadata = additional_metadata.get(_build_url(data["href"]), {})
-                            data["docs_blob_url"] = metadata.get("docs_blob_url")
+                            _add_collection_level_metadata(data, additional_metadata)
+                            _add_collection_version_level_metadata(data, additional_metadata)
                             yield data
-                            progress_bar.increment()
 
 
 class DocsBlobDownloader(ArtifactDownloader):
@@ -453,6 +534,11 @@ class CollectionContentSaver(ContentSaver):
     Saves Collection and Tag objects related to the CollectionVersion content unit.
     """
 
+    def __init__(self, repository_version, *args, **kwargs):
+        """Initialize CollectionContentSaver."""
+        super().__init__(*args, **kwargs)
+        self.repository_version = repository_version
+
     async def _pre_save(self, batch):
         """
         Save a batch of Collection objects.
@@ -462,6 +548,8 @@ class CollectionContentSaver(ContentSaver):
                 :class:`~pulpcore.plugin.stages.DeclarativeContent` objects to be saved.
 
         """
+        to_deprecate = []
+
         for d_content in batch:
             if d_content is None:
                 continue
@@ -472,8 +560,19 @@ class CollectionContentSaver(ContentSaver):
             collection, created = Collection.objects.get_or_create(
                 namespace=info["namespace"], name=info["name"]
             )
+
             d_content.content.collection = collection
 
+            if d_content.extra_data.get("deprecated"):
+                to_deprecate.append(
+                    AnsibleCollectionDeprecated(
+                        repository_version=self.repository_version, collection=collection
+                    )
+                )
+
+        if to_deprecate:
+            AnsibleCollectionDeprecated.objects.bulk_create(to_deprecate, ignore_conflicts=True)
+
     async def _post_save(self, batch):
         """
         Save a batch of CollectionVersion, Tag objects.
