@@ -5,17 +5,18 @@ from packaging.version import parse
 from django.contrib.postgres.search import SearchQuery
 from django.db.models import fields as db_fields
 from django.db.models.expressions import F, Func
-from django_filters import filters, MultipleChoiceFilter
+from django_filters import filters
 from django_filters.rest_framework import DjangoFilterBackend
 from drf_spectacular.utils import extend_schema
 from rest_framework import mixins, serializers, viewsets
 from rest_framework.decorators import action
 from rest_framework.filters import OrderingFilter
 from rest_framework.parsers import FormParser, MultiPartParser
+from rest_framework.serializers import ValidationError as DRFValidationError
 
 from pulpcore.plugin.actions import ModifyRepositoryActionMixin
 from pulpcore.plugin.exceptions import DigestValidationError
-from pulpcore.plugin.models import PulpTemporaryFile
+from pulpcore.plugin.models import PulpTemporaryFile, RepositoryVersion
 from pulpcore.plugin.serializers import (
     AsyncOperationResponseSerializer,
     RepositorySyncURLSerializer,
@@ -51,10 +52,12 @@ from .serializers import (
     CollectionVersionSerializer,
     CollectionRemoteSerializer,
     CollectionOneShotSerializer,
+    CopySerializer,
     RoleSerializer,
     TagSerializer,
 )
 from .tasks.collections import sync as collection_sync
+from .tasks.copy import copy_content
 from .tasks.roles import synchronize as role_sync
 
 
@@ -111,8 +114,6 @@ class CollectionVersionFilter(ContentFilter):
     namespace = filters.CharFilter(field_name="namespace")
     name = filters.CharFilter(field_name="name")
     is_highest = filters.BooleanFilter(field_name="is_highest", method="get_highest")
-    certification = MultipleChoiceFilter(choices=CollectionVersion.CERTIFICATION_CHOICES)
-    deprecated = filters.BooleanFilter(field_name="collection__deprecated")
     q = filters.CharFilter(field_name="q", method="filter_by_q")
     tags = filters.CharFilter(
         field_name="tags",
@@ -162,20 +163,9 @@ class CollectionVersionFilter(ContentFilter):
 
     def get_highest(self, qs, name, value):
         """
-        Combine certification and is_highest filters.
+        Filter queryset qs by is_highest filter.
 
-        If certification and is_highest are used together,
-        get the highest version for the specified certification
         """
-        certification = self.data.get("certification")
-
-        if not certification:
-            return qs.filter(is_highest=value)
-
-        qs = qs.filter(certification=certification)
-        if not qs.count():
-            return qs
-
         latest_pks = []
         namespace_name_dict = defaultdict(lambda: defaultdict(list))
         for collection in qs.all():
@@ -192,7 +182,7 @@ class CollectionVersionFilter(ContentFilter):
 
     class Meta:
         model = CollectionVersion
-        fields = ["namespace", "name", "version", "q", "is_highest", "certification", "tags"]
+        fields = ["namespace", "name", "version", "q", "is_highest", "tags"]
 
 
 class CollectionVersionViewSet(ContentViewSet):
@@ -242,8 +232,8 @@ class AnsibleRepositoryViewSet(RepositoryViewSet, ModifyRepositoryActionMixin):
         )
         serializer.is_valid(raise_exception=True)
 
-        remote = serializer.validated_data.get("remote")
-        remote.cast()
+        remote = serializer.validated_data.get("remote", repository.remote)
+        remote = remote.cast()
 
         if isinstance(remote, RoleRemote):
             sync_func = role_sync
@@ -341,3 +331,70 @@ class TagViewSet(NamedModelViewSet, mixins.ListModelMixin):
     endpoint_name = "pulp_ansible/tags"
     queryset = Tag.objects.all()
     serializer_class = TagSerializer
+
+
+class CopyViewSet(viewsets.ViewSet):
+    """
+    ViewSet for Content Copy.
+    """
+
+    serializer_class = CopySerializer
+
+    @extend_schema(
+        description="Trigger an asynchronous task to copy ansible content from one repository "
+        "into another, creating a new repository version.",
+        summary="Copy content",
+        operation_id="copy_content",
+        request=CopySerializer,
+        responses={202: AsyncOperationResponseSerializer},
+    )
+    def create(self, request):
+        """Copy content."""
+        serializer = CopySerializer(data=request.data, context={"request": request})
+        serializer.is_valid(raise_exception=True)
+
+        config = serializer.validated_data["config"]
+
+        config, repos = self._process_config(config)
+
+        async_result = enqueue_with_reservation(copy_content, repos, args=[config], kwargs={})
+        return OperationPostponedResponse(async_result, request)
+
+    def _process_config(self, config):
+        """
+        Change the hrefs into pks within config.
+
+        This method also implicitly validates that the hrefs map to objects and it returns a list of
+        repos so that the task can lock on them.
+        """
+        result = []
+        repos = []
+
+        for entry in config:
+            r = dict()
+            source_version = NamedModelViewSet().get_resource(
+                entry["source_repo_version"], RepositoryVersion
+            )
+            dest_repo = NamedModelViewSet().get_resource(entry["dest_repo"], AnsibleRepository)
+            r["source_repo_version"] = source_version.pk
+            r["dest_repo"] = dest_repo.pk
+            repos.extend((source_version.repository, dest_repo))
+
+            if "dest_base_version" in entry:
+                try:
+                    r["dest_base_version"] = dest_repo.versions.get(
+                        number=entry["dest_base_version"]
+                    ).pk
+                except RepositoryVersion.DoesNotExist:
+                    message = _(
+                        "Version {version} does not exist for repository " "'{repo}'."
+                    ).format(version=entry["dest_base_version"], repo=dest_repo.name)
+                    raise DRFValidationError(detail=message)
+
+            if entry.get("content") is not None:
+                r["content"] = []
+                for c in entry["content"]:
+                    r["content"].append(NamedModelViewSet().extract_pk(c))
+            result.append(r)
+
+        return result, repos
