@@ -170,10 +170,6 @@ def test_collection_detail(artifact, collection_detail, pulp_dist):
         pulp_dist["base_path"], f"/v3/collections/{artifact.namespace}/{artifact.name}/"
     )
 
-    # Detail Endpoint
-    assert "created_at" in collection_detail
-    assert "updated_at" in collection_detail
-
     assert not collection_detail["deprecated"]
     assert collection_detail["href"] == url
     assert collection_detail["namespace"] == artifact.namespace
@@ -189,7 +185,6 @@ def test_collection_version_list(artifact, pulp_client, collection_detail):
     version = versions["data"][0]
 
     assert version["version"] == "1.0.0"
-    assert version["certification"] == "needs_review"
     assert version["href"] == collection_detail["highest_version"]["href"]
 
 
@@ -206,7 +201,6 @@ def test_collection_version(artifact, pulp_client, collection_detail):
     assert version["name"] == artifact.name
     assert version["namespace"] == {"name": artifact.namespace}
     assert version["version"] == "1.0.0"
-    assert version["certification"] == "needs_review"
 
     tarball = open(artifact.filename, "rb").read()
     assert version["artifact"]["sha256"] == hashlib.sha256(tarball).hexdigest()
